'-------------------------------------------------------------------------------
'name                     : Si570_FWrite01
'copyright                : (c) 2008, Dipl.Ing. Dr. Christian Hirt, Austria
'purpose                  : Si570 New Frequency Write TEST
'micro                    : AT90S2313
'flash                    : 73% used
'build                    : 24032008 (24. Mar. 2008)
'status                   : tested (non-warranty!)
'-------------------------------------------------------------------------------

$regfile = "2313def.dat"                'AT90S2313
$crystal = 3686400                      'crystal frequency
$hwstack = 32                           'hardware stack   32
$swstack = 10                           'SW stack         10
$framesize = 40                         'frame space      40

Config Scl = Portb.0                    'I2C config
Config Sda = Portb.1

Const AddrSi570 = &h55                  'SI570 I2C slave base address
Const Si570WAdr = &hAA                  'I2C write byte
Const Si570RAdr = &hAB                  'I2C read  byte

' Config LCD
Config Lcdbus = 4
Config Lcd = 16 * 1a
Config Lcdmode = Port
Config Lcdpin = Pin , Rs = Portb.2 , E = Portb.3
Config Lcdpin = Pin , Db4 = Portb.4 , Db5 = Portb.5 , Db6 = Portb.6 , Db7 = Portb.7

Declare Sub Write_si570(byval Addr As Byte , Byval Value As Byte)
Declare Function Read_si570(byval Addr As Byte) As Byte
Declare Sub WriteNewRegMap()
Declare Sub LoadNewFreq()

Dim HSDIV As Byte
Dim N1 As Byte
Dim RFREQ As Long
Dim i As Byte
Dim a As Byte
Dim reg(6) As Byte

'---- Main ---------------------------------------------------------------------

'-------------------------------
'StartUpFreq = 56.3200000 MHz
'InitHSDIV = 4
'InitN1 = 22
'InitRFEQ = 43.22936620 MHz
'
'New Frequency = 100.0 MHz
'
'==> Calculate dividers and reference frequency multiplier
'==> Excel program
'==>
HSDIV = 5                               'HSDIV = {4,5,6,7,9,11}
N1 = 10                                 'N1 = {1,2,4,6...128}
'RFREQ = 43.61175417
'==>
RFREQ = &b1010_1110_0111_0010_0110_1111_1010_1111
'-------------------------------

Cursor Off
Cls
Lcd "Si570 - WR Test"
Wait 2
Cls

Call WriteNewRegMap()                   'generate new register map

Call LoadNewFreq()                      'load values to Si570

Lcd "New Frequency"

End                                     'End of program

'-------------------------------------------------------------------------------

Sub WriteNewRegMap()                    'Write new register map
   Dim N11 As Byte , N1a As Byte , N1b As Byte
   Dim Fr1 As Long , Fr2 As Long , Fr3 As Long , Fr4 As Long , Fr5 As Long

   For i = 1 To 6
      reg(i) = 0                        'Clear register map
   Next i

   'new HS_DIV conversion
   HSDIV = HSDIV - 4
   Shift HSDIV , Left , 5
   reg(1) = HSDIV                       'set register 7 bits 5, 6, 7

   'new N1 conversion
   If N1 = 1 Then
      N1 = 0
   Else
      N11 = N1 And 1
      If N11 = 0 Then
         N1 = N1 - 1                    'if N1 is even, subtract one
      End If
   End If
   N1a = N1
   Shift N1a , Right , 2
   reg(1) = reg(1) Or N1a               'set register 7 bits 0 to 4
   N1b = N1 And &b0000_0011
   Shift N1b , Left , 6
   reg(2) = N1b                         'set register 8 bits 6 and 7

   'new RFREQ conversion
   Fr1 = RFREQ
   Shift Fr1 , Right , 30
   reg(2) = reg(2) Or Fr1               'set register 8 bits 0 - 5
   Fr2 = RFREQ
   Shift Fr2 , Right , 22
   reg(3) = Fr2                         'set register 9
   Fr3 = RFREQ
   Shift Fr3 , Right , 14
   reg(4) = Fr3                         'set register 10
   Fr4 = RFREQ
   Shift Fr4 , Right , 6
   reg(5) = Fr4                         'set register 11
   Fr5 = RFREQ
   Shift Fr5 , Left , 2
   reg(6) = Fr5                         'set register 12
End Sub

'-------------------------------------------------------------------------------

Sub LoadNewFreq()                       'Load the new frequency
   Dim Reg137 As Byte
   Dim Reg135 As Byte

   Reg137 = Read_Si570(137)             'get current state of register 137
   Reg137 = Reg137 Or &b0001_0000
   Call Write_Si570(137 , Reg137)       'freeze DCO

   For i = 1 To 6                       'load new values into registers
     a = i + 6
     Call Write_Si570(a , reg(i))
   Next i

   Reg137 = Read_Si570(137)             'get the current state

   Reg137 = Reg137 And &b1110_1111
   Call Write_Si570(137 , Reg137)       'clear Freeze DCO bit

   Call Write_Si570(135 , &b0100_0000)  'set the NewFreq bit
End Sub

'--- I2C functions -------------------------------------------------------------

Sub Write_Si570(byval Addr As Byte , Byval Value As Byte)       'write a byte
   I2cstart                             'start condition
   I2cwbyte Si570wadr                   'write slave address from Si570
   I2cwbyte Addr                        'adress of Si570 register
   I2cwbyte Value                       'byte value to write
   I2cstop                              'stop condition
End Sub

Function Read_Si570(byval Addr As Byte) As Byte       'read a byte from Si570
  Dim Rdval As Byte
  Rdval = 0
  I2cstart                              'start condition
  I2cwbyte Si570WAdr                    'write slave adress from Si570
  I2cwbyte Addr                         'address of Si570 register
  I2cstart                              'repeated start
  I2cwbyte Si570RAdr                    'slave address (read) from Si570
  I2crbyte Rdval , Nack                 'read byte
  I2cstop                               'generate stop
  Read_Si570 = Rdval
End Function


'------------------------ End of Si570_FWrite01 --------------------------------