'-------------------------------------------------------------------------------
'name                     : Si570_SUC_Test_01
'copyright                : (c) 2008, Dipl.Ing. Dr. Christian Hirt, Austria
'purpose                  : Si570 StartUp Configuration TEST
'micro                    : AT90S2313
'flash                    : 76% used
'build                    : 23032008 (23. Mar. 2008)
'status                   : tested (non-warranty!)
'-------------------------------------------------------------------------------

$regfile = "2313def.dat"                'AT90S2313
$crystal = 3686400                      'crystal frequency
$hwstack = 32                           'hardware stack   32
$swstack = 10                           'SW stack         10
$framesize = 40                         'frame space      40

Config Scl = Portb.0                    'I2C config
Config Sda = Portb.1

Const AddrSi570 = &h55                  'SI570 I2C slave base address
Const Si570WAdr = &hAA                  'I2C write byte
Const Si570RAdr = &hAB                  'I2C read  byte

' Config LCD
Config Lcdbus = 4
Config Lcd = 16 * 1a
Config Lcdmode = Port
Config Lcdpin = Pin , Rs = Portb.2 , E = Portb.3
Config Lcdpin = Pin , Db4 = Portb.4 , Db5 = Portb.5 , Db6 = Portb.6 , Db7 = Portb.7

Declare Sub ReadStartupConfig()
Declare Sub Write_Si570(Byval Addr As Byte , Byval Value As Byte)
Declare Function Read_Si570(Byval Addr As Byte) As Byte

Dim InitHSDIV As Byte
Dim InitN1 As Byte
Dim InitRFREQ As Long
Dim i As Byte
Dim a As Byte
Dim reg(6) As Byte
Dim intx As Byte
Dim fracx As Long
'---- Main ---------------------------------------------------------------------

Cursor Off
Cls
Lcd "Si570 - SUC Test"
Wait 2
Cls
Call ReadStartupConfig()                'read Si570 data
Lcd InitHSDIV                           'show HSDIV
Wait 5
Cls
Lcd InitN1                              'show N1
Wait 5
Cls
intx = High(InitRFREQ) AND &b11111100
shift intx , right , 2
fracx = InitRFREQ AND &b0000_0011_1111_1111_1111_1111_1111_1111
Lcd intx ; "," ; fracx                  'show InitFREQ

End

'###########################################
'RESULT for this Si570
'  Measurement => FREQ = 56.321.540 Hz
'  Nominal FREQ => 56.320.000 Hz
'
'  InitHSDIV: 4
'  InitN1:    22
'  InitRFREQ: 43.22936620
'###########################################

'-------------------------------------------------------------------------------

Sub ReadStartupConfig()                 'Read Si570 startup configuration
   Dim Initn1a As Byte , Initn1b As Byte

   Call Write_Si570(135 , &b0000_0001)  'Recall the initial state in NVM

   For i = 1 To 6
      a = i + 6
      reg(i) = Read_Si570(a)            'Read registers 7 to 12 of Si570
   Next i

   'HS_DIV conversion
   InitHSDIV = reg(1) And &b1110_0000
   Shift InitHSDIV , Right , 5
   InitHSDIV = InitHSDIV + 4            'HS_DIV = {4,5,6,7,9,11}

   'N1 conversion
   Initn1a = Reg(1) And &b0001_1111
   Shift Initn1a , Left , 2
   Initn1b = reg(2) And &b1100_0000
   Shift Initn1b , Right , 6
   InitN1 = Initn1a + Initn1b           'N1 = {1,2,4,6...128}
   If Initn1 = 0 Then
      InitN1 = 1
   Else
      Initn1a = InitN1 And &b0000_0001
      If Initn1a <> 0 Then
         InitN1 = InitN1 + 1
      End If
   End If

   'RFREQ conversion
   'Cut the 6 MSB bits and ignore lowest 2 bits to fit into LONG format!
   'The data type LONG is stored as 32-bit binary number (4 bytes)
   InitRFREQ = reg(2) And &b0011_1111
   Shift Initrfreq , Left , 8
   InitRFREQ = InitRFREQ + reg(3)
   Shift InitRFREQ , Left , 8
   InitRFREQ = InitRFREQ + reg(4)
   Shift InitRFREQ , Left , 8
   InitRFREQ = InitRFREQ + reg(5)
   Shift InitRFREQ , Left , 6
   Shift reg(6) , Right , 2
   InitRFREQ = InitRFREQ + reg(6)
End Sub


'--- I2C functions -------------------------------------------------------------

Sub Write_Si570(byval Addr As Byte , Byval Value As Byte)       'write a byte
   I2cstart                             'start condition
   I2cwbyte Si570wadr                   'write slave address from Si570
   I2cwbyte Addr                        'adress of Si570 register
   I2cwbyte Value                       'byte value to write
   I2cstop                              'stop condition
End Sub

Function Read_Si570(byval Addr As Byte) As Byte       'read a byte from Si570
  Dim Rdval As Byte
  Rdval = 0
  I2cstart                              'start condition
  I2cwbyte Si570WAdr                    'write slave adress from Si570
  I2cwbyte Addr                         'address of Si570 register
  I2cstart                              'repeated start
  I2cwbyte Si570RAdr                    'slave address (read) from Si570
  I2crbyte Rdval , Nack                 'read byte
  I2cstop                               'generate stop
  Read_si570 = Rdval
End Function

'-------------------------- End of Si570_SUC_Test_01 ---------------------------